NetworkController = util.singleton( "NetworkController" )
NetworkController.ServerProtocol = class( "Server", lube.Server )
NetworkController.ServerProtocol._implemented = true
NetworkController.PORT = 27015
NetworkController.HANDSHAKE = "locofilm.hex.0.3"
NetworkController.playerid = 0
NetworkController.uniquePlayerid = 0

function NetworkController:update( dt )
    if self.server then
        self.server:update( dt )
    end
    if self.client then
        self.client:update( dt )
    end
end

function NetworkController:serializeAndCompressTable( t )
    local s  = TSerial.pack( t )
    local sc = TLibCompress.CompressLZW( s )
    return sc
end

function NetworkController:decompressAndDeserializeData( data )
    local s = TLibCompress.DecompressLZW( data )
    local t = TSerial.unpack( s )
    return t
end


-------------------------
-- SERVER
-------------------------
function NetworkController:startServer()
    self.server = lube.tcpServer:new()
    self.server:listen( self.PORT )
    self.server:setoption( "tcp-nodelay", true )
    self.server.handshake               = self.HANDSHAKE
    self.server.callbacks.recv          = NetworkController.onServerReceiveData
    self.server.callbacks.connect       = NetworkController.onServerClientConnect
    self.server.callbacks.disconnect    = NetworkController.onServerClientDisconnect
    self.server.numberOfClientsConnected = 0
end

function NetworkController:getUniquePlayerID()
    self.uniquePlayerid = self.uniquePlayerid + 1
    return self.uniquePlayerid
end

-- Callbacks
function NetworkController.onServerReceiveData( data, clientid )
    NetworkController.server:send( data )
end

function NetworkController.onServerClientConnect( clientid )
    print( "Client connected." )
    NetworkController.server.numberOfClientsConnected = NetworkController.server.numberOfClientsConnected + 1
    local playerid = NetworkController:getUniquePlayerID()
    NetworkController:sendTableToClient { ['_m'] = MSG_CLIENT_CONNECTED, playerid }
    NetworkController:sendTableToClient( { 
        ['_m'] = MSG_SNAPSHOT,
        playerid,
        MessageController.turn,
        NetworkController.server.numberOfClientsConnected,
        RNG.seed
    }, clientid )
end

function NetworkController.onServerClientDisconnect( clientid )
    print( "Client disconnected." )
    NetworkController.server.numberOfClientsConnected = NetworkController.server.numberOfClientsConnected - 1
    NetworkController.server:send( { ['_m'] = MSG_CLIENT_DISCONNECTED } )
end

function NetworkController:sendTableToClient( t, client )
    local data = self:serializeAndCompressTable( t )
    data = "\002" .. data .. "\003"
    self.server:send( data, client )
end

-------------------------
-- CLIENT
-------------------------
function NetworkController:startClient( ip, playername )
    self.client = lube.tcpClient:new()
    self.client.handshake = self.HANDSHAKE
    local success, err = self.client:connect( ip, self.PORT, true )
    if not success then error(err) end
    
    self.client:setoption( "tcp-nodelay", true )
    self.client.callbacks.recv = NetworkController.onClientReceiveData
end

function NetworkController.onClientReceiveData( data )
    for m in string.gmatch( data, "\002([^\003]*)" ) do
        if m then
            local message = NetworkController:decompressAndDeserializeData( m )
            MessageController:onReceiveMessage( message )
        end
    end
end

function NetworkController:sendDataToServer( data )
    if self.client then
        local success, err = self.client:send( data )
        if not success then error(err) end
    end
end

function NetworkController:sendTableToServer( t )
    if self.client then
        local data = self:serializeAndCompressTable( t )
        data = "\002" .. data .. "\003"
        local success, err = self.client:send( data )
        if not success then error( err ) end
    end
end

function NetworkController:receivedPlayerID( playerid )
    self.playerid = playerid
end