PortRenderer = class('PortRender', EntityRender)

function PortRenderer:initialize()
    --EntityRender.initialize(self)
end

function PortRenderer:setPortController( portController )
    self.portController = portController
end

-- This is called from the system renderer so we are already translated to the system position.
function PortRenderer:render()
    local position = self.portController:getPosition()
    local boundingBox = self.portController.boundingBox
    local systemPos = self.portController.systemController.position
    
    love.graphics.push()
    love.graphics.translate( position.x, position.y )
    --
    if self.portController.inputController:getMouseOverState() == MOUSE_OVER then
        love.graphics.setColor( 0, 255, 0 )
    else
        love.graphics.setColor( 220, 30, 30 )
    end
    love.graphics.circle( "fill", 0, 0, 16, 32 )
    --
    love.graphics.pop()
    
    -- If the port is awaiting selection of a forward port then display a line to the mouse.
    if self.portController.inputController:isAwaitingForwardPortSelection() and GridController.instance:getSystemUnderMouse() == self.portController.systemController then
        local worldMousePos = getCurrentCamera():mousepos()
        local x, y = self.portController.systemController:worldToLocalCoords( worldMousePos.x, worldMousePos.y )
        if self.portController:forwardLaneIntersectsWithOtherForwardLanes( vector(position.x, position.y), vector(x, y) ) then
            love.graphics.setColor( 255, 0, 0 )
        else
            love.graphics.setColor( 255, 255, 255 )
        end
        love.graphics.line( position.x, position.y, x, y )
    end
end

function PortRenderer:renderLine()
    local position = self.portController:getPosition()
    local sisterPort = self.portController:getSisterPort()
    local x1, y1 = self.portController.systemController:localToWorldCoords( position.x, position.y )
    local x2, y2 = sisterPort.systemController:localToWorldCoords(sisterPort.position.x, sisterPort.position.y)

    local cameraZoom = getCurrentCamera().zoom
    
    -- draw a line to the sister port.
    if self.portController.isMaster then
        if self.portController:isBackdoor() then
            love.graphics.setColor( 125, 125, 125 )
        elseif self.portController:isForwarded() or (self.portController:isConnected() and self.portController.sisterPort:isForwarded()) then
            love.graphics.setColor( 255, 120, 120 )
        else
            love.graphics.setColor( 205, 205, 205 )
        end
        love.graphics.setLineWidth( math.max( 1, 1 / cameraZoom) )
        love.graphics.line( x1, y1, x2, y2 )
    end

    love.graphics.setColor(18,203,250)
    for i, packet in ipairs(self.portController.packets) do
        local t1 = math.max( 0, (packet.transferProgress * 0.01) )
        local t2 = math.min( 1, (packet.transferProgress * 0.01) + 0.07 )
        local pos1 = vector.lerp( vector(x1,y1), vector(x2,y2), t1 )
        local pos2 = vector.lerp( vector(x1,y1), vector(x2,y2), t2 )
        local r = math.max( 10, 3 * (1 / cameraZoom))
        love.graphics.line( pos1.x, pos1.y, pos2.x, pos2.y )
        --love.graphics.circle( "fill", pos.x, pos.y, r )
    end
    
    -- If the port is forwarded, draw a line to the forwarded port.
    if self.portController:isForwarded() and self.portController:isForwardMaster() then
        local forwardPortPos = self.portController:getForwardPort():getPosition()
        local x, y = self.portController.systemController:localToWorldCoords( forwardPortPos.x, forwardPortPos.y )
        love.graphics.setColor( 255, 120, 120 )
        love.graphics.line( x1, y1, x, y )
    end
end