require 'InputController'

ProgramInputController = class('ProgramInputController', InputController)
ProgramInputController:include( Stateful )
ProgramInputController.STATE_INSIDE_PORT = ProgramInputController:addState('INSIDE_PORT')

function ProgramInputController:initialize()
    InputController.initialize(self)
end

function ProgramInputController:setProgramController( programController )
    self.programController = programController
end

function ProgramInputController:programEnteredState( stateName )
    if stateName == 'INSIDE_PORT' then
        self:gotoState( 'INSIDE_PORT' )
    end
end

function ProgramInputController:programExitedState( stateName )
    if stateName == 'INSIDE_PORT' then
        self:gotoState()
    end
end

function ProgramInputController.STATE_INSIDE_PORT:enteredState()
    local tags = {"input", self}
    --NotificationController.killTags( tags, TAGS_ALL )
    --NotificationController.listen("on_mouse_button_pressed", tags, ProgramInputController.STATE_INSIDE_PORT.onMouseButtonPressed, self)
end

function ProgramInputController.STATE_INSIDE_PORT:exitedState()
    local tags = {"input", self}
   -- NotificationController.killTags( tags, TAGS_ALL )
    --NotificationController.listen("on_mouse_button_pressed", tags, self.onMouseButtonPressed, self)
end

function ProgramInputController.STATE_INSIDE_PORT:update(dt)
end

function ProgramInputController:update( dt )
    if self.programController:getState() ~= ProgramController.STATE_INSIDE_PORT then
        if GridController.instance:getSystemUnderMouse() == self.programController.systemController then
            local p = getCurrentCamera():localMouseCoords( self.programController.systemController.position )
            local boundingBox = self.programController:getBoundingBox()
            self:updateMouseDownState( p, boundingBox, 'l', 'm', 'r' )
        else
            self.mouseOverState = MOUSE_OFF
        end
    end
end

function ProgramInputController.STATE_INSIDE_PORT:onMouseButtonPressed( mx, my, button )
    if button == 'l' then
        self.programController.selected = false
    end
end

function ProgramInputController:onMouseButtonPressed( mx, my, button )
    local p = getCurrentCamera():localMouseCoords( self.programController.systemController.position )
    if self.programController:getSelected() and self.mouseOverState == MOUSE_OFF then
        if GridController.instance:isSystemUnderMouse( self.programController.systemController ) then
            local entityUnderMouse = self:getEntityUnderMouse( ProgramController, PortController )
            if button == 'r' then 
                if not entityUnderMouse then
                    self.programController:sendMessage( MSG_PROGRAM_ORDER_MOVE, p.x, p.y )
                elseif instanceOf( PortController, entityUnderMouse ) then
                    self.programController:sendMessage( MSG_PROGRAM_ORDER_GOAL_PORT, entityUnderMouse )
                elseif self.programController:isEnemyWith( entityUnderMouse ) then
                    self.programController:sendMessage( MSG_PROGRAM_ORDER_ATTACK, entityUnderMouse )
                elseif self.programController:isAlliedWith( entityUnderMouse ) then
                    self.programController:sendMessage( MSG_PROGRAM_ORDER_ASSIST, entityUnderMouse )
                end
            end
            if button == 'l' then
                if not programUnderMouse then
                    NotificationController.send("deselect_all")
                end
            end
        else
            local programUnderMouse = self:getEntityUnderMouse( ProgramController )
            if button == 'r' and programUnderMouse then
                if self.programController:isEnemyWith( programUnderMouse ) then
                    self.programController:sendMessage( MSG_PROGRAM_ORDER_ATTACK, programUnderMouse )
                end
            elseif button == 'r' then
                local systemUnderMouse = GridController.instance:getSystemUnderMouse()
                if systemUnderMouse then
                    self.programController:sendMessage( MSG_PROGRAM_ORDER_GOAL_SYSTEM, systemUnderMouse )
                end
            elseif button == 'l' then
                self.programController.selected = false
            end
        end
    end
end

function ProgramInputController:onMouseButtonReleased( mx, my, button )
end

function ProgramInputController:onEnterMouseOff( )
end

function ProgramInputController:onEnterMouseOver( )
end

function ProgramInputController:onEnterMouseDown( )
    if love.mouse.isDown('l') then
        local entityUnderMouse = self:getEntityUnderMouse( ProgramController )
        if entityUnderMouse == self.programController then
            self.programController:setSelected( true, true )
        end
    end
end

