StateMainMenu = Gamestate.new()

function StateMainMenu:init( oldState, ... )
    self.fontBackgroundTitle = love.graphics.newFont( FONTS_PATH .. "Agency.ttf", 100 )
end

function StateMainMenu:enter( oldState, ... )
    local frame = loveframes.Create( "frame" )
    frame:SetSize(300,320)
    frame:SetName("")
    frame:ShowCloseButton(false)
    frame:Center()
    self.frameMain = frame
    
    local buttonList = loveframes.Create( "list", self.frameMain )
    buttonList:SetSize(frame:GetWidth(), frame:GetHeight() - 25)
	buttonList:SetPos(0, 25)
	buttonList:SetPadding(10)
	buttonList:SetSpacing(20)
	buttonList:SetDisplayType("vertical")
	
    local buttons = {
        {"Create Game", "CreateGame" },
        {"Join Game", "JoinGame" },
        {"Options", "Options" },
        {"Quit", "Quit" },
    }
    for i, buttonInfo in ipairs(buttons) do
        local button = loveframes.Create( "button" )
        button:SetHeight( 50 )
        button:SetText( buttonInfo[1] )
        local callback = "on" .. buttonInfo[2] .. "ButtonClick"
        if self[callback] then
            button.OnClick = self[callback]
        end
        buttonList:AddItem( button )
    end
end

function StateMainMenu:leave()
    self.frameMain:Remove()
end

function StateMainMenu.onCreateGameButtonClick( button )
    Gamestate.switch( StateGame, true )
end

function StateMainMenu.onJoinGameButtonClick( button )
    StateMainMenu.frameMain:SetVisible( false )
    local frame = loveframes.Create( "frame" )
    frame:SetSize( 400, 240 )
    frame:SetName( "Join Game" )
    frame:Center()
    frame.OnClose = function(object)
        StateMainMenu.frameMain:SetVisible( true )
        StateMainMenu.frameMain:ShowCloseButton(false)
    end
    
    local list = loveframes.Create( "list", frame )
    list:SetSize( frame:GetWidth(), frame:GetHeight() - 25 )
    list:SetPos( 0, 25 )
    list:SetPadding(15)
    list:SetSpacing(10)
    list:SetDisplayType("vertical")
    
    local ipLabel = loveframes.Create( "text" )
    ipLabel:SetText( "Enter the IP address of the game you wish to connect to.")
    
    local nameLabel = loveframes.Create( "text" )
    nameLabel:SetText( "Enter the name you wish to connect as.")
    
    local ipInput = loveframes.Create( "textinput" )
    ipInput:SetFocus( true )
    
    local nameInput = loveframes.Create( "textinput" )
    
    local joinButton = loveframes.Create( "button" )
    joinButton:SetHeight( 50 )
    joinButton:SetText( "Connect to Game" )
    joinButton:Center()
    joinButton.OnClick = function()
        StateMainMenu:connectToGame( ipInput.text, nameInput.text )
    end
    
    list:AddItem( ipLabel )
    list:AddItem( ipInput )
    list:AddItem( nameLabel )
    list:AddItem( nameInput )
    list:AddItem( joinButton )
end

function StateMainMenu.onOptionsButtonClick( button )
    StateMainMenu.frameMain:SetVisible( false )
    local frame = loveframes.Create( "frame" )
    frame:SetSize( 400, 500 )
    frame:SetName( "Options" )
    frame:Center()
    frame.OnClose = function(object)
        StateMainMenu.frameMain:SetVisible( true )
        StateMainMenu.frameMain:ShowCloseButton(false)
    end
    
    local list = loveframes.Create( "list", frame )
    list:SetSize( frame:GetWidth(), frame:GetHeight() - 25 )
    list:SetPos( 0, 25 )
    list:SetPadding(15)
    list:SetSpacing(10)
    list:SetDisplayType("vertical")
    
    local modeLabel = loveframes.Create( "text" )
    modeLabel:SetText( "Select a resolution" )
    
    list:AddItem( modeLabel )
    
    local supportedModes = love.graphics.getModes()
    for i, mode in ipairs(supportedModes) do
        local button = loveframes.Create( "button" )
        button:SetText(mode.width .. " x " .. mode.height)
        button.OnClick = function(b)
            local width, height, fullscreen, vsync, fsaa = love.graphics.getMode( )
            love.graphics.setMode( mode.width, mode.height, fullscreen, vsync, fsaa )
            frame:Center()
            StateMainMenu.frameMain:Center()
        end
        list:AddItem(button)
    end
    
    local fullscreenLabel = loveframes.Create( "text" )
    fullscreenLabel:SetText( "Select fullscreen option" )
    
    local fullscreenButton = loveframes.Create("button")
    fullscreenButton:SetText( "Set Fullscreen" )
    fullscreenButton.OnClick = function(b)
        local width, height, fullscreen, vsync, fsaa = love.graphics.getMode( )
        love.graphics.setMode( width, height, true, vsync, fsaa )
        frame:Center()
        StateMainMenu.frameMain:Center()
    end
    
    local windowedButton = loveframes.Create("button")
    windowedButton:SetText( "Set Windowed" )
    windowedButton.OnClick = function(b)
        local width, height, fullscreen, vsync, fsaa = love.graphics.getMode( )
        love.graphics.setMode( width, height, false, vsync, fsaa )
        frame:Center()
        StateMainMenu.frameMain:Center()
    end
    
    list:AddItem(fullscreenLabel)
    list:AddItem(fullscreenButton)
    list:AddItem(windowedButton)
end

function StateMainMenu.onQuitButtonClick( button )
    love.event.quit()
end

function StateMainMenu:connectToGame( ip, name )
    Gamestate.switch( StateGame, false, ip, name )
end

function StateMainMenu:update( dt )
end

function StateMainMenu:draw()
    love.graphics.setBackgroundColor( 50, 50, 50 )
    love.graphics.setFont( self.fontBackgroundTitle )
    love.graphics.print( "HEX.", 20, 0 )
    
    loveframes.draw()
end