require 'Hexagon'

SystemRender = class('SystemRender', EntityRender)

function SystemRender:initialize( systemController, hexagonShape, systemSpacing )
    EntityRender.initialize( self )
    self.systemController = systemController
    self.hexagonShape = hexagonShape
    self.position = systemController.position:clone()
    self.systemIsInViewport = false
    self.centerX, self.centerY = self.hexagonShape:getCenter()
end

function SystemRender:render()
    -- Check if any of the systems vertices are inside the viewport.
    local currentCamera = getCurrentCamera()
    
    self.systemIsInViewport = false
    local pos = self.position
    local hexvertices = self.hexagonShape.vertices
    for i = 1, #hexvertices, 2 do
        local x, y = hexvertices[i], hexvertices[i+1]
        if currentCamera:isPointInsideViewport( pos.x + x, pos.y + y, 200 ) then
            self.systemIsInViewport = true
            break
        end
    end
    
    -- Render if the system appears in the viewport.
    if self.systemIsInViewport then
        if self.systemController.isEven then
            love.graphics.setColor(100,100,100)
        else
            love.graphics.setColor( 160, 160, 160 )
        end
    
        if GridController.instance:getSystemUnderMouse() == self.systemController then love.graphics.setColor(100,100,200) end
        self.hexagonShape:render( self.position.x, self.position.y )
        --love.graphics.print(self.systemController.gridPosition.i..", "..self.systemController.gridPosition.j, self.position.x, self.position.y)
    
        love.graphics.setColor(255,255,255)
        love.graphics.push()
        love.graphics.translate( self.position.x, self.position.y )
        --
        if currentCamera.zoom >= 1/10 then
            for i, port in ipairs(self.systemController.ports) do
                port.portRenderer:render()
            end
            for i, program in ipairs(self.systemController.programs) do
                program.programRenderer:render()
            end
        end
        --
        love.graphics.pop()
    end
end

--[[
TODO: draw circuit lines from cpu to any point in system.
Proposed method:
- project 3 lines, each 30 degrees apart, from point.
- project a line from the cpu.
- find the intersection point.
- draw a line from cpu to intersection point and the from that to the target.
]]
local _line0deg = vector( 0, 1 )
local _line60deg = vector.rotate2D( vector( 0, 1 ), math.rad(60) )
local _line120deg = vector.rotate2D( vector( 0, 1 ), math.rad(120) )
function SystemRender:renderCircuitLine()
    if GridController.instance:getSystemUnderMouse() ~= self.systemController then return end
    local mp = getCurrentCamera():mousepos()
    local px, py = mp.x - self.position.x, mp.y - self.position.y
    local cx, cy = self.hexagonShape:getCenter()
    local ix, iy = util.lineIntersection( cx, cy, cx + 1, cy + 0, px, py, px + _line60deg.x, py + _line60deg.y )
    if ix then
        love.graphics.line( cx, cy, ix, iy )
        love.graphics.line( px, py, ix, iy )
    end
end