require 'libs.middleclass'

util = {}

-- constants
DIR_N   = 0
DIR_NW  = 1
DIR_SW  = 2
DIR_S   = 3
DIR_SE  = 4
DIR_NE  = 5
--

function util.reverseHexDirection( direction )
    return (direction + 3) % 6
end

function util.lerpXY( x1, y1, x2, y2, t )
    local x3 = (x2 - x1) * t
    local y3 = (y2 - y1) * t
    return x1 + x3, y1 + y3
end

function util.lerp( a, b, t )
    return a * ( 1 - t ) + b * t
end

function util.reverseXY( x1, y1, x2, y2 )
    return x2, y2, x1, y1
end

function table.shuffle( t )
    for i = #t, 2, -1 do
        local dice = RNG:random(i)
        t[i], t[dice] = t[dice], t[i]
    end
end

function util.singleton( name, superclass )
    local _newClass = class( name, superclass )
    return _newClass:new()
end

-- http://wiki.processing.org/w/Line-Line_intersection
function util.lineIntersection(x1, y1, x2, y2, x3, y3, x4, y4)
  local bx = x2 - x1
  local by = y2 - y1
  local dx = x4 - x3
  local dy = y4 - y3 
  local b_dot_d_perp = bx*dy - by*dx
  if b_dot_d_perp == 0 then
    return nil
  end
  local cx = x3-x1
  local cy = y3-y1
  local t = (cx*dy - cy*dx) / b_dot_d_perp
 
  return x1+t*bx, y1+t*by
end

function util.segIntersection(x1, y1, x2, y2, x3, y3, x4, y4) 
  local bx = x2 - x1 
  local by = y2 - y1 
  local dx = x4 - x3 
  local dy = y4 - y3
  local b_dot_d_perp = bx * dy - by * dx
  if b_dot_d_perp == 0 then
    return nil
  end
  local cx = x3 - x1
  local cy = y3 - y1
  local t = (cx * dy - cy * dx) / b_dot_d_perp
  if (t < 0) or (t > 1) then
    return nil
  end
  local u = (cx * by - cy * bx) / b_dot_d_perp;
  if (u < 0 or u > 1) then 
    return nil
  end
  return x1+t*bx, y1+t*by
end

return util