require 'libs.TSerial'
require 'libs.TLibCompress'

describe["When serializing a table"] = function()
    before = function()
        aTable = {
            ['_p'] = 1,
            ['_t'] = 2,
            { ['_m'] = 1, 2, 3 },
            { ['_m'] = 1, 2, 3 },
            { ['_m'] = 1, 2, 3 },
            { ['_m'] = 1, 2, 3 },
        }
        serializedTable = TSerial.pack( aTable )
    end

    it["should be a string"] = function()
        expect( serializedTable ).is_type( "string" )
    end
    
    describe["when deserialized"] = function()
        before = function()
            deserializedTable = TSerial.unpack( serializedTable )
        end
        
        it["should be a table"] = function()
            expect( deserializedTable ).is_type( "table" )
        end
        
        it["should have the correct keys and values"] = function()
            expect( deserializedTable ).should_equal( aTable )
        end
    end
    
    describe["when compressed"] = function()
        before = function()
            serializedAndCompressedTable = TLibCompress.CompressLZW( serializedTable )
        end
        
        it["should be a string"] = function()
            expect( serializedAndCompressedTable ).is_type( "string" )
        end
        
        it["should be smaller or equal in size than the serialized table"] = function()
            print(string.len(serializedAndCompressedTable))
            print(string.len(serializedTable))
            expect( string.len(serializedAndCompressedTable) <= string.len( serializedTable ) ).should_be( true )
        end
        
        describe["and then decompressed"] = function()
            before = function()
                serializedAndDecompressedTable = TLibCompress.DecompressLZW( serializedAndCompressedTable )
            end
            
            it["should be a string"] = function()
                expect( serializedAndDecompressedTable ).is_type( "string" )
            end
            
            it["should be larger or equal in size to the compressed string"] = function()
                local isLargerOrEqual = string.len( serializedAndDecompressedTable ) >= string.len( serializedAndCompressedTable )
                expect( isLargerOrEqual ).should_be( true )
            end
            
            describe["and then deserialized"] = function()
                before = function()
                    deserializedAndDecompressedTable = TSerial.unpack( serializedAndDecompressedTable )
                end
                
                it["should be a table"] = function()
                    expect( deserializedAndDecompressedTable ).is_type( "table" )
                end
                
                it["should have the correct keys and values"] = function()
                    expect( deserializedAndDecompressedTable ).should_equal( aTable )
                end
            end
        end
    end
end